use crate::{
    soundclips,
    state::{AppState, TemplateEngine},
    vbplay,
};
use axum::extract::State;
use axum_template::{Key, RenderHtml};

use log::info;
use serde::Serialize;

#[derive(Serialize)]
pub struct TemplateContext {
    clips: Vec<soundclips::SoundClip>,
}

pub async fn home(
    engine: TemplateEngine,
    state: State<AppState>,
) -> impl axum::response::IntoResponse {
    let clips = state.0.clone().clips; // this is not ideal.
    let context = TemplateContext { clips: clips };
    RenderHtml(Key("soundboard.jinja".to_owned()), engine, context)
}

pub async fn play_handler(
    axum::extract::Path(hash): axum::extract::Path<String>,
    state: State<AppState>,
) -> String {
    if let Some(clip) = state
        .clips
        .clone()
        .into_iter()
        .find(|s| hash == s.hash().to_string())
    {
        let player = state.player.clone();
        let full_file_name = &clip.full_file_name();
        info!("Playing {}", full_file_name);
        player
            .lock()
            .unwrap()
            .listener
            .send(vbplay::Command::PlayWhilePressing(
                full_file_name.into(),
                "".to_owned(), // placeholder.
            ))
            .unwrap();
    }
    "".to_owned()
}

pub async fn stop_handler(state: State<AppState>) -> String {
    let player = state.player.clone();
    player
        .lock()
        .unwrap()
        .listener
        .send(vbplay::Command::Stop)
        .unwrap();

    "".to_owned()
}
